"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var AbstractCategoryLogger_1 = require("./AbstractCategoryLogger");
var LoggerOptions_1 = require("./LoggerOptions");
/**
 * Simple logger, that logs to the console. If the console is unavailable will throw an exception.
 */
var CategoryConsoleLoggerImpl = (function (_super) {
    __extends(CategoryConsoleLoggerImpl, _super);
    function CategoryConsoleLoggerImpl(rootCategory, runtimeSettings) {
        return _super.call(this, rootCategory, runtimeSettings) || this;
    }
    CategoryConsoleLoggerImpl.prototype.doLog = function (msg) {
        var fullMsg = this.createDefaultLogMessage(msg);
        if (console !== undefined) {
            var logged = false;
            /* tslint:disable:no-console */
            switch (msg.getLevel()) {
                case LoggerOptions_1.LogLevel.Trace:
                    if (console.trace) {
                        console.trace(fullMsg);
                        logged = true;
                    }
                    break;
                case LoggerOptions_1.LogLevel.Debug:
                    if (console.debug) {
                        console.debug(fullMsg);
                        logged = true;
                    }
                    break;
                case LoggerOptions_1.LogLevel.Info:
                    if (console.info) {
                        console.info(fullMsg);
                        logged = true;
                    }
                    break;
                case LoggerOptions_1.LogLevel.Warn:
                    if (console.warn) {
                        console.warn(fullMsg);
                        logged = true;
                    }
                    break;
                case LoggerOptions_1.LogLevel.Error:
                case LoggerOptions_1.LogLevel.Fatal:
                    if (console.error) {
                        console.error(fullMsg);
                        logged = true;
                    }
                    break;
                default:
                    throw new Error("Unsupported level: " + msg.getLevel());
            }
            if (!logged) {
                console.log(fullMsg);
            }
        }
        else {
            throw new Error("Console is not defined, cannot log msg: " + fullMsg);
        }
    };
    return CategoryConsoleLoggerImpl;
}(AbstractCategoryLogger_1.AbstractCategoryLogger));
exports.CategoryConsoleLoggerImpl = CategoryConsoleLoggerImpl;
//# sourceMappingURL=CategoryConsoleLoggerImpl.js.map