"use strict";
var LoggerImpl_1 = require("./LoggerImpl");
var LoggerOptions_1 = require("./LoggerOptions");
var LoggerFactoryImpl = (function () {
    function LoggerFactoryImpl(options) {
        this.loggers = {};
        this.configure(options);
    }
    LoggerFactoryImpl.prototype.configure = function (options) {
        this.options = options;
        // Close any current open loggers.
        this.closeLoggers();
    };
    LoggerFactoryImpl.prototype.getLogger = function (named) {
        if (!this.options.enabled) {
            throw new Error("LoggerFactory is not enabled, please check your options passed in");
        }
        var logger = this.loggers[named];
        if (logger !== undefined) {
            return logger;
        }
        // Initialize logger with appropriate level
        logger = this.loadLogger(named);
        this.loggers[named] = logger;
        return logger;
    };
    LoggerFactoryImpl.prototype.isEnabled = function () {
        return this.options.enabled;
    };
    LoggerFactoryImpl.prototype.closeLoggers = function () {
        for (var key in this.loggers) {
            if (this.loggers.hasOwnProperty(key)) {
                this.loggers[key].close();
            }
        }
        this.loggers = {};
    };
    LoggerFactoryImpl.prototype.loadLogger = function (named) {
        var logGroupRules = this.options.logGroupRules;
        for (var _i = 0, logGroupRules_1 = logGroupRules; _i < logGroupRules_1.length; _i++) {
            var logGroupRule = logGroupRules_1[_i];
            if (logGroupRule.regExp.test(named)) {
                switch (logGroupRule.loggerType) {
                    case LoggerOptions_1.LoggerType.Console:
                        return new LoggerImpl_1.ConsoleLoggerImpl(named, logGroupRule);
                    case LoggerOptions_1.LoggerType.MessageBuffer:
                        return new LoggerImpl_1.MessageBufferLoggerImpl(named, logGroupRule);
                    case LoggerOptions_1.LoggerType.Custom:
                        if (logGroupRule.callBackLogger != null) {
                            return logGroupRule.callBackLogger(named, logGroupRule);
                        }
                        else {
                            throw new Error("Cannot create a custom logger, custom callback is null");
                        }
                    default:
                        throw new Error("Cannot create a Logger for LoggerType: " + logGroupRule.loggerType);
                }
            }
        }
        throw new Error("Failed to find a match to create a Logger for: " + named);
    };
    return LoggerFactoryImpl;
}());
exports.LoggerFactoryImpl = LoggerFactoryImpl;
//# sourceMappingURL=LoggerFactoryImpl.js.map